/**
 * @license
 * Copyright (c) 2017 The Polymer Project Authors. All rights reserved.
 * This code may only be used under the BSD style license found at
 * http://polymer.github.io/LICENSE.txt
 * The complete set of authors may be found at
 * http://polymer.github.io/AUTHORS.txt
 * The complete set of contributors may be found at
 * http://polymer.github.io/CONTRIBUTORS.txt
 * Code distributed by Google as part of the polymer project is also
 * subject to an additional IP rights grant found at
 * http://polymer.github.io/PATENTS.txt
 */
/** TemplateResult types */
declare const HTML_RESULT = 1;
declare const SVG_RESULT = 2;
/** TemplatePart types */
export declare const ATTRIBUTE_PART = 1;
export declare const NODE_PART = 2;
export declare const PROPERTY_PART = 3;
export declare const BOOLEAN_ATTRIBUTE_PART = 4;
export declare const EVENT_PART = 5;
declare type ResultType = typeof HTML_RESULT | typeof SVG_RESULT;
/**
 * The return type of the template tag functions.
 */
export declare type TemplateResult = {
    _$litType$: ResultType;
    strings: TemplateStringsArray;
    values: unknown[];
};
/**
 * Interprets a template literal as an HTML template that can efficiently
 * render to and update a container.
 */
export declare const html: (strings: TemplateStringsArray, ...values: unknown[]) => TemplateResult;
/**
 * Interprets a template literal as an SVG template that can efficiently
 * render to and update a container.
 */
export declare const svg: (strings: TemplateStringsArray, ...values: unknown[]) => TemplateResult;
/**
 * A sentinel value that signals that a value was handled by a directive and
 * should not be written to the DOM.
 */
export declare const noChange: {};
/**
 * A sentinel value that signals a NodePart to fully clear its content.
 */
export declare const nothing: {};
export declare type NodePartInfo = {
    readonly type: typeof NODE_PART;
};
export declare type AttributePartInfo = {
    readonly type: typeof ATTRIBUTE_PART | typeof PROPERTY_PART | typeof BOOLEAN_ATTRIBUTE_PART | typeof EVENT_PART;
    strings?: ReadonlyArray<string>;
    name: string;
    tagName: string;
};
/**
 * Information about the part a directive is bound to.
 *
 * This is useful for checking that a directive is attached to a valid part,
 * such as with directive that can only be used on attribute bindings.
 */
export declare type PartInfo = NodePartInfo | AttributePartInfo;
export declare type DirectiveClass = {
    new (part: PartInfo): Directive;
};
/**
 * This utility type extracts the signature of a directive class's render()
 * method so we can use it for the type of the generated directive function.
 */
export declare type DirectiveParameters<C extends DirectiveClass> = Parameters<InstanceType<C>['render']>;
/**
 * A generated directive function doesn't evaluate the directive, but just
 * returns a DirectiveResult object that captures the arguments.
 */
declare type DirectiveResult<C extends DirectiveClass = DirectiveClass> = {
    _$litDirective$: C;
    values: DirectiveParameters<C>;
};
/**
 * Creates a user-facing directive function from a Directive class. This
 * function has the same parameters as the directive's render() method.
 *
 * WARNING: The directive and part API changes are in progress and subject to
 * change in future pre-releases.
 */
export declare const directive: <C extends DirectiveClass>(c: C) => (...values: Parameters<InstanceType<C>["render"]>) => DirectiveResult<C>;
export interface RenderOptions {
    /**
     * An object to use as the `this` value for event listeners. It's often
     * useful to set this to the host component rendering a template.
     */
    readonly eventContext?: EventTarget;
    /**
     * A DOM node before which to render content in the container.
     */
    readonly renderBefore?: ChildNode | null;
}
/**
 * Renders a value, usually a lit-html TemplateResult, to the container.
 * @param value
 * @param container
 * @param options
 */
export declare const render: (value: unknown, container: HTMLElement | DocumentFragment, options?: RenderOptions | undefined) => void;
/**
 * Base class for creating custom directives. Users should extend this class,
 * implement `render` and/or `update`, and then pass their subclass to
 * `directive`.
 *
 * WARNING: The directive and part API changes are in progress and subject to
 * change in future pre-releases.
 */
export declare abstract class Directive {
    abstract render(...props: Array<unknown>): unknown;
    update(_part: Part, props: Array<unknown>): unknown;
}
export declare type Part = NodePart | AttributePart | PropertyPart | BooleanAttributePart;
export declare class NodePart {
    _startNode: ChildNode;
    _endNode: ChildNode | null;
    options: RenderOptions | undefined;
    readonly type = 2;
    _value: unknown;
    protected __directive?: Directive;
    constructor(_startNode: ChildNode, _endNode: ChildNode | null, options: RenderOptions | undefined);
    _setValue(value: unknown): void;
    private __insert;
    private __commitDirective;
    private _commitNode;
    private __commitText;
    private __commitTemplateResult;
    private __commitIterable;
    __clear(start?: ChildNode | null): void;
}
export declare class AttributePart {
    readonly type: 1 | 5 | 4 | 3;
    readonly element: HTMLElement;
    readonly name: string;
    /**
     * If this attribute part represents an interpolation, this contains the
     * static strings of the interpolation. For single-value, complete bindings,
     * this is undefined.
     */
    readonly strings?: ReadonlyArray<string>;
    _value: unknown | Array<unknown>;
    private __directives?;
    get tagName(): string;
    constructor(element: HTMLElement, name: string, strings: ReadonlyArray<string>, _options?: RenderOptions);
    /**
     * Normalizes a user-provided value before writing it to the DOM. In the
     * near future this will include invoking a directive if the value is
     * a DirectiveResult.
     *
     * @param value the raw input value to normalize
     * @param _i the index in the values array this value was read from
     */
    __resolveValue(value: unknown, i: number): unknown;
    /**
     * Sets the value of this part.
     *
     * If this part is single-valued, `this.__strings` will be undefined, and the
     * method will be called with a single value argument. If this part is
     * multi-value, `this.__strings` will be defined, and the method is called
     * with the value array of the part's owning TemplateInstance, and an offset
     * into the value array from which the values should be read.
     *
     * This method is overloaded this way to eliminate short-lived array slices
     * of the template instance values, and allow a fast-path for single-valued
     * parts.
     *
     * @param value The part value, or an array of values for multi-valued parts
     * @param from the index to start reading values from. `undefined` for
     *   single-valued parts
     */
    _setValue(value: unknown): void;
    _setValue(value: Array<unknown>, from: number): void;
    /**
     * Writes the value to the DOM. An override point for PropertyPart and
     * BooleanAttributePart.
     */
    __commitValue(value: unknown): void;
}
export declare class PropertyPart extends AttributePart {
    readonly type = 3;
    __commitValue(value: unknown): void;
}
export declare class BooleanAttributePart extends AttributePart {
    readonly type = 4;
    __commitValue(value: unknown): void;
}
/**
 * An AttributePart that manages an event listener via add/removeEventListener.
 *
 * This part works by adding itself as the event listener on an element, then
 * delegating to the value passed to it. This reduces the number of calls to
 * add/removeEventListener if the listener changes frequently, such as when an
 * inline function is used as a listener.
 *
 * Because event options are passed when adding listeners, we must take case
 * to add and remove the part as a listener when the event options change.
 */
export declare class EventPart extends AttributePart {
    readonly type = 5;
    __eventContext?: unknown;
    constructor(...args: ConstructorParameters<typeof AttributePart>);
    _setValue(newListener: unknown): void;
    handleEvent(event: Event): void;
}
export {};
//# sourceMappingURL=lit-html.d.ts.map