/**
 * @license
 * Copyright (c) 2017 The Polymer Project Authors. All rights reserved.
 * This code may only be used under the BSD style license found at
 * http://polymer.github.io/LICENSE.txt
 * The complete set of authors may be found at
 * http://polymer.github.io/AUTHORS.txt
 * The complete set of contributors may be found at
 * http://polymer.github.io/CONTRIBUTORS.txt
 * Code distributed by Google as part of the polymer project is also
 * subject to an additional IP rights grant found at
 * http://polymer.github.io/PATENTS.txt
 */
var _a;
var _b;
const DEV_MODE = true;
if (DEV_MODE) {
    console.warn('lit-html is in dev mode. Not recommended for production!');
}
// Added to an attribute name to mark the attribute as bound so we can find
// it easily.
const boundAttributeSuffix = '$lit$';
// This marker is used in many syntactic positions in HTML, so it must be
// a valid element name and attribute name. We don't support dynamic names (yet)
// but this at least ensures that the parse tree is closer to the template
// intention.
const marker = `lit$${String(Math.random()).slice(9)}$`;
// String used to tell if a comment is a marker comment
const markerMatch = '?' + marker;
// Text used to insert a comment marker node. We use processing instruction
// syntax because it's slightly smaller, but parses as a comment node.
const nodeMarker = `<${markerMatch}>`;
const d = document;
// Creates a dynamic marker. We never have to search for these in the DOM.
const createMarker = (v = '') => d.createComment(v);
const isPrimitive = (value) => value === null || (typeof value != 'object' && typeof value != 'function');
const isArray = Array.isArray;
const isIterable = (value) => isArray(value) ||
    (value && typeof value[Symbol.iterator] === 'function');
// TODO (justinfagnani): can we get away with `\s`?
const SPACE_CHAR = `[ \t\n\f\r]`;
const ATTR_VALUE_CHAR = `[^ \t\n\f\r"'\`<>=]`;
const NAME_CHAR = `[^\0-\x1F\x7F-\x9F "'>=/]`;
// These regexes represent the five parsing states that we care about in the
// Template's HTML scanner. They match the *end* of the state they're named
// after.
// Depending on the match, we transition to a new state. If there's no match,
// we stay in the same state.
// Note that the regexes are stateful. We utilize lastIndex and sync it
// across the multiple regexes used. In addition to the five regexes below
// we also dynamically create a regex to find the matching end tags for raw
// text elements.
// TODO (justinfagnani): we detect many more parsing edge-cases than we
// used to, and many of those are of dubious value. Decide and document
// how to relax correctness to simplify the regexes and states.
/**
 * End of text is: `<` followed by:
 *   (comment start) or (tag) or (dynamic tag binding)
 */
const textEndRegex = /<(?:(!--|\/[^a-zA-Z])|(\/?[a-zA-Z][^>\s]*)|(\/?$))/g;
const COMMENT_START = 1;
const TAG_NAME = 2;
const DYNAMIC_TAG_NAME = 3;
const commentEndRegex = /-->/g;
/**
 * Comments not started with <!--, like </{, can be ended by a single `>`
 */
const comment2EndRegex = />/g;
/**
 * The tagEnd regex matches the end of the "inside an opening" tag syntax
 * position. It either matches a `>` or an attribute.
 *
 * See attributes in the HTML spec:
 * https://www.w3.org/TR/html5/syntax.html#elements-attributes
 *
 * " \t\n\f\r" are HTML space characters:
 * https://infra.spec.whatwg.org/#ascii-whitespace
 *
 * "\0-\x1F\x7F-\x9F" are Unicode control characters, which includes every
 * space character except " ".
 *
 * So an attribute is:
 *  * The name: any character except a control character, space character, ('),
 *    ("), ">", "=", or "/"
 *  * Followed by zero or more space characters
 *  * Followed by "="
 *  * Followed by zero or more space characters
 *  * Followed by:
 *    * Any character except space, ('), ("), "<", ">", "=", (`), or
 *    * (") then any non-("), or
 *    * (') then any non-(')
 */
const tagEndRegex = new RegExp(`>|${SPACE_CHAR}(${NAME_CHAR}+)(${SPACE_CHAR}*=${SPACE_CHAR}*(?:${ATTR_VALUE_CHAR}|("|')|))`, 'g');
const ENTIRE_MATCH = 0;
const ATTRIBUTE_NAME = 1;
const SPACES_AND_EQUALS = 2;
const QUOTE_CHAR = 3;
const singleQuoteAttrEndRegex = /'/g;
const doubleQuoteAttrEndRegex = /"/g;
/**
 * Matches the raw text elements.
 *
 * Comments are not parsed within raw text elements, so we need to search their
 * text content for marker strings.
 */
const rawTextElement = /^(?:script|style|textarea)$/i;
/** TemplateResult types */
const HTML_RESULT = 1;
const SVG_RESULT = 2;
/** TemplatePart types */
// TODO (justinfagnani): since these are exported, consider shorter names,
// like just `ATTRIBUTE`.
export const ATTRIBUTE_PART = 1;
export const NODE_PART = 2;
export const PROPERTY_PART = 3;
export const BOOLEAN_ATTRIBUTE_PART = 4;
export const EVENT_PART = 5;
const ELEMENT_PART = 6;
const COMMENT_PART = 7;
/**
 * Generates a template literal tag function that returns a TemplateResult with
 * the given result type.
 */
const tag = (_$litType$) => (strings, ...values) => ({
    _$litType$,
    strings,
    values,
});
/**
 * Interprets a template literal as an HTML template that can efficiently
 * render to and update a container.
 */
export const html = tag(HTML_RESULT);
/**
 * Interprets a template literal as an SVG template that can efficiently
 * render to and update a container.
 */
export const svg = tag(SVG_RESULT);
/**
 * A sentinel value that signals that a value was handled by a directive and
 * should not be written to the DOM.
 */
export const noChange = {};
/**
 * A sentinel value that signals a NodePart to fully clear its content.
 */
export const nothing = {};
/**
 * The cache of prepared templates, keyed by the tagged TemplateStringsArray
 * and _not_ accounting for the specific template tag used. This means that
 * template tags cannot be dynamic - the must statically be one of html, svg,
 * or attr. This restriction simplifies the cache lookup, which is on the hot
 * path for rendering.
 */
const templateCache = new Map();
/**
 * Creates a user-facing directive function from a Directive class. This
 * function has the same parameters as the directive's render() method.
 *
 * WARNING: The directive and part API changes are in progress and subject to
 * change in future pre-releases.
 */
export const directive = (c) => (...values) => ({
    _$litDirective$: c,
    values,
});
/**
 * Renders a value, usually a lit-html TemplateResult, to the container.
 * @param value
 * @param container
 * @param options
 */
export const render = (value, container, options) => {
    var _a, _b;
    const partOwnerNode = (_a = options === null || options === void 0 ? void 0 : options.renderBefore) !== null && _a !== void 0 ? _a : container;
    let part = partOwnerNode.$lit$;
    if (part === undefined) {
        const endNode = (_b = options === null || options === void 0 ? void 0 : options.renderBefore) !== null && _b !== void 0 ? _b : null;
        partOwnerNode.$lit$ = part = new NodePart(container.insertBefore(createMarker(), endNode), endNode, options);
    }
    part._setValue(value);
};
const walker = d.createTreeWalker(d);
//
// Classes only below here, const variable declarations only above here...
//
// Keeping variable declarations and classes together improves minification.
// Interfaces and type aliases can be interleaved freely.
//
/**
 * Base class for creating custom directives. Users should extend this class,
 * implement `render` and/or `update`, and then pass their subclass to
 * `directive`.
 *
 * WARNING: The directive and part API changes are in progress and subject to
 * change in future pre-releases.
 */
export class Directive {
    update(_part, props) {
        return this.render(...props);
    }
}
class Template {
    constructor({ strings, _$litType$: type }) {
        this.__parts = [];
        walker.currentNode = (this.__element = d.createElement('template')).content;
        // Insert makers into the template HTML to represent the position of
        // bindings. The following code scans the template strings to determine the
        // syntactic position of the bindings. They can be in text position, where
        // we insert an HTML comment, attribute value position, where we insert a
        // sentinel string and re-write the attribute name, or inside a tag where
        // we insert the sentinel string.
        const l = (this.__strings = strings).length - 1;
        const attrNames = [];
        let html = type === SVG_RESULT ? '<svg>' : '';
        let node;
        let nodeIndex = 0;
        let bindingIndex = 0;
        let attrNameIndex = 0;
        // When we're inside a raw text tag (not it's text content), the regex
        // will still be tagRegex so we can find attributes, but will switch to
        // this regex when the tag ends.
        let rawTextEndRegex;
        // The current parsing state, represented as a reference to one of the
        // regexes
        let regex = textEndRegex;
        for (let i = 0; i < l; i++) {
            const s = strings[i];
            // The index of the end of the last attribute name. When this is
            // positive at end of a string, it means we're in an attribute value
            // position and need to rewrite the attribute name.
            let attrNameEndIndex = -1;
            let attrName;
            let lastIndex = 0;
            let match;
            // The conditions in this loop handle the current parse state, and the
            // assignments to the `regex` variable are the state transitions.
            while (lastIndex < s.length) {
                // Make sure we start searching from where we previously left off
                regex.lastIndex = lastIndex;
                match = regex.exec(s);
                if (match === null) {
                    // If the current regex doesn't match we've come to a binding inside
                    // that state and must break and insert a marker
                    if (regex === tagEndRegex) {
                        // When tagEndRegex doesn't match we must have a binding in
                        // attribute-name position, since tagEndRegex does match static
                        // attribute names and end-of-tag. We need to clear
                        // attrNameEndIndex which may have been set by a previous
                        // tagEndRegex match.
                        attrNameEndIndex = -1;
                    }
                    break;
                }
                lastIndex = regex.lastIndex;
                if (regex === textEndRegex) {
                    if (match[COMMENT_START] === '!--') {
                        regex = commentEndRegex;
                    }
                    else if (match[COMMENT_START] !== undefined) {
                        // We started a weird comment, like </{
                        regex = comment2EndRegex;
                    }
                    else if (match[TAG_NAME] !== undefined) {
                        if (rawTextElement.test(match[TAG_NAME])) {
                            // Record if we encounter a raw-text element. We'll switch to
                            // this regex at the end of the tag
                            rawTextEndRegex = new RegExp(`<\/${match[TAG_NAME]}`, 'g');
                        }
                        regex = tagEndRegex;
                    }
                    else if (match[DYNAMIC_TAG_NAME] !== undefined) {
                        // dynamic tag name
                        regex = tagEndRegex;
                    }
                }
                else if (regex === tagEndRegex) {
                    if (match[ENTIRE_MATCH] === '>') {
                        // End of a tag. If we had started a raw-text element, use that
                        // regex
                        regex = rawTextEndRegex !== null && rawTextEndRegex !== void 0 ? rawTextEndRegex : textEndRegex;
                        // We may be ending an unquoted attribute value, so make sure we
                        // clear any pending attrNameEndIndex
                        attrNameEndIndex = -1;
                    }
                    else {
                        attrNameEndIndex =
                            regex.lastIndex - match[SPACES_AND_EQUALS].length;
                        attrName = match[ATTRIBUTE_NAME];
                        regex =
                            match[QUOTE_CHAR] === undefined
                                ? tagEndRegex
                                : match[QUOTE_CHAR] === '"'
                                    ? doubleQuoteAttrEndRegex
                                    : singleQuoteAttrEndRegex;
                    }
                }
                else if (regex === doubleQuoteAttrEndRegex ||
                    regex === singleQuoteAttrEndRegex) {
                    regex = tagEndRegex;
                }
                else if (regex === commentEndRegex || regex === comment2EndRegex) {
                    regex = textEndRegex;
                }
                else {
                    // Not one of the five state regexes, so it must be the dynamically
                    // created raw text regex and we're at the close of that element.
                    regex = tagEndRegex;
                    rawTextEndRegex = undefined;
                }
            }
            if (DEV_MODE) {
                // If we have a attrNameEndIndex, which indicates that we should
                // rewrite the attribute name, assert that we're in a valid attribute
                // position - either in a tag, or a quoted attribute value.
                console.assert(attrNameEndIndex === -1 ||
                    regex === tagEndRegex ||
                    regex === singleQuoteAttrEndRegex ||
                    regex === doubleQuoteAttrEndRegex, 'unexpected parse state B');
            }
            // If we're in text position, and not in a raw text element
            // (regex === textEndRegex), we insert a comment marker. Otherwise, we
            // insert a plain maker. If we have a attrNameEndIndex, it means we need
            // to rewrite the attribute name to add a bound attribute suffix.
            html +=
                regex === textEndRegex
                    ? s + nodeMarker
                    : (attrNameEndIndex !== -1
                        ? (attrNames.push(attrName),
                            s.slice(0, attrNameEndIndex) +
                                boundAttributeSuffix +
                                s.slice(attrNameEndIndex))
                        : s) + marker;
        }
        // TODO (justinfagnani): if regex is not textRegex log a warning for a
        // malformed template in dev mode.
        // Note, we don't add '</svg>' for SVG result types because the parser
        // will close the <svg> tag for us.
        this.__element.innerHTML = html + this.__strings[l];
        if (type === SVG_RESULT) {
            const content = this.__element.content;
            const svgElement = content.firstChild;
            svgElement.remove();
            content.append(...svgElement.childNodes);
        }
        // Walk the template to find binding markers and create TemplateParts
        while ((node = walker.nextNode()) !== null && bindingIndex < l) {
            if (node.nodeType === 1) {
                // TODO (justinfagnani): for attempted dynamic tag names, we don't
                // increment the bindingIndex, and it'll be off by 1 in the element
                // and off by two after it.
                if (node.hasAttributes()) {
                    const { attributes } = node;
                    for (let i = 0; i < attributes.length; i++) {
                        const { name, value } = attributes[i];
                        if (name.endsWith(boundAttributeSuffix)) {
                            i--;
                            node.removeAttribute(name);
                            const statics = value.split(marker);
                            const m = /([.?@])?(.*)/.exec(attrNames[attrNameIndex++]);
                            this.__parts.push({
                                __type: ATTRIBUTE_PART,
                                __index: nodeIndex,
                                __name: m[2],
                                __strings: statics,
                                __constructor: m[1] === '.'
                                    ? PropertyPart
                                    : m[1] === '?'
                                        ? BooleanAttributePart
                                        : m[1] === '@'
                                            ? EventPart
                                            : AttributePart,
                            });
                            bindingIndex += statics.length - 1;
                        }
                        else if (name === marker) {
                            node.removeAttribute(name);
                            i--;
                            this.__parts.push({
                                __type: ELEMENT_PART,
                                __index: nodeIndex,
                            });
                        }
                    }
                }
                // TODO (justinfagnani): benchmark the regex against testing for each
                // of the 3 raw text element names.
                if (rawTextElement.test(node.tagName)) {
                    // For raw text elements we need to split the text content on
                    // markers, create a Text node for each segment, and create
                    // a TemplatePart for each marker.
                    const strings = node.textContent.split(marker);
                    const lastIndex = strings.length - 1;
                    if (lastIndex > 0) {
                        node.textContent = '';
                        // Generate a new text node for each literal section
                        // These nodes are also used as the markers for node parts
                        // We can't use empty text nodes as markers because they're
                        // normalized in some browsers (TODO: check)
                        for (let i = 0; i < lastIndex; i++) {
                            node.append(strings[i] || createMarker());
                            this.__parts.push({ __type: NODE_PART, __index: ++nodeIndex });
                            bindingIndex++;
                        }
                        node.append(strings[lastIndex] || createMarker());
                    }
                }
            }
            else if (node.nodeType === 8) {
                const data = node.data;
                if (data === markerMatch) {
                    bindingIndex++;
                    this.__parts.push({ __type: NODE_PART, __index: nodeIndex });
                }
                else {
                    let i = -1;
                    while ((i = node.data.indexOf(marker, i + 1)) !== -1) {
                        // Comment node has a binding marker inside, make an inactive part
                        // The binding won't work, but subsequent bindings will
                        // TODO (justinfagnani): consider whether it's even worth it to
                        // make bindings in comments work
                        this.__parts.push({ __type: COMMENT_PART, __index: nodeIndex });
                        bindingIndex++;
                        // Move to the end of the match
                        i += marker.length - 1;
                    }
                }
            }
            nodeIndex++;
        }
    }
}
/**
 * An updateable instance of a Template. Holds references to the Parts used to
 * update the template instance.
 */
class TemplateInstance {
    constructor(template) {
        this.__parts = [];
        this.__template = template;
    }
    // This method is separate from the constructor because we need to return a
    // DocumentFragment and we don't want to hold onto it with an instance field.
    __clone(options) {
        const { __element: { content }, __parts: parts, } = this.__template;
        const fragment = d.importNode(content, true);
        walker.currentNode = fragment;
        let node = walker.nextNode();
        let nodeIndex = 0;
        let partIndex = 0;
        let templatePart = parts[0];
        while (templatePart !== undefined && node !== null) {
            if (nodeIndex === templatePart.__index) {
                let part;
                if (templatePart.__type === NODE_PART) {
                    part = new NodePart(node, node.nextSibling, options);
                }
                else if (templatePart.__type === ATTRIBUTE_PART) {
                    part = new templatePart.__constructor(node, templatePart.__name, templatePart.__strings, options);
                }
                this.__parts.push(part);
                templatePart = parts[++partIndex];
            }
            if (templatePart !== undefined && nodeIndex !== templatePart.__index) {
                node = walker.nextNode();
                nodeIndex++;
            }
        }
        return fragment;
    }
    __update(values) {
        let i = 0;
        for (const part of this.__parts) {
            if (part === undefined) {
                i++;
                continue;
            }
            if (part.strings !== undefined) {
                part._setValue(values, i);
                i += part.strings.length - 1;
            }
            else {
                part._setValue(values[i++]);
            }
        }
    }
}
export class NodePart {
    constructor(_startNode, _endNode, options) {
        this._startNode = _startNode;
        this._endNode = _endNode;
        this.options = options;
        this.type = NODE_PART;
    }
    _setValue(value) {
        // TODO (justinfagnani): when setting a non-directive over a directive,
        // we don't yet clear this.__directive.
        // See https://github.com/Polymer/lit-html/issues/1286
        if (isPrimitive(value)) {
            if (value !== this._value) {
                this.__commitText(value);
            }
        }
        else if (value._$litType$ !== undefined) {
            this.__commitTemplateResult(value);
        }
        else if (value._$litDirective$ !== undefined) {
            this.__commitDirective(value);
        }
        else if (value.nodeType !== undefined) {
            this._commitNode(value);
        }
        else if (isIterable(value)) {
            this.__commitIterable(value);
        }
        else if (value === nothing) {
            this._value = nothing;
            this.__clear();
        }
        else if (value !== noChange) {
            // Fallback, will render the string representation
            this.__commitText(value);
        }
    }
    __insert(node, ref = this._endNode) {
        return this._startNode.parentNode.insertBefore(node, ref);
    }
    __commitDirective(value) {
        var _a;
        const directive = value._$litDirective$;
        if (((_a = this.__directive) === null || _a === void 0 ? void 0 : _a.constructor) !== directive) {
            this.__clear();
            this.__directive = new directive(this);
        }
        // TODO (justinfagnani): To support nested directives, we'd need to
        // resolve the directive result's values. We may want to offer another
        // way of composing directives.
        this._setValue(this.__directive.update(this, value.values));
    }
    _commitNode(value) {
        if (this._value !== value) {
            this.__clear();
            this._value = this.__insert(value);
        }
    }
    __commitText(value) {
        const node = this._startNode.nextSibling;
        // Make sure undefined and null render as an empty string
        // TODO: use `nothing` to clear the node?
        value !== null && value !== void 0 ? value : (value = '');
        // TODO(justinfagnani): Can we just check if this._value is primitive?
        if (node !== null &&
            node.nodeType === 3 /* Node.TEXT_NODE */ &&
            (this._endNode === null
                ? node.nextSibling === null
                : node === this._endNode.previousSibling)) {
            // If we only have a single text node between the markers, we can just
            // set its value, rather than replacing it.
            node.data = value;
        }
        else {
            this._commitNode(new Text(value));
        }
        this._value = value;
    }
    __commitTemplateResult(result) {
        const { strings, values } = result;
        let template = templateCache.get(strings);
        if (template === undefined) {
            templateCache.set(strings, (template = new Template(result)));
        }
        if (this._value != null &&
            this._value.__template === template) {
            this._value.__update(values);
        }
        else {
            const instance = new TemplateInstance(template);
            const fragment = instance.__clone(this.options);
            instance.__update(values);
            this._commitNode(fragment);
            this._value = instance;
        }
    }
    __commitIterable(value) {
        // For an Iterable, we create a new InstancePart per item, then set its
        // value to the item. This is a little bit of overhead for every item in
        // an Iterable, but it lets us recurse easily and efficiently update Arrays
        // of TemplateResults that will be commonly returned from expressions like:
        // array.map((i) => html`${i}`), by reusing existing TemplateInstances.
        // If value is an array, then the previous render was of an
        // iterable and value will contain the NodeParts from the previous
        // render. If value is not an array, clear this part and make a new
        // array for NodeParts.
        if (!isArray(this._value)) {
            this._value = [];
            this.__clear();
        }
        // Lets us keep track of how many items we stamped so we can clear leftover
        // items from a previous render
        const itemParts = this._value;
        let partIndex = 0;
        let itemPart;
        for (const item of value) {
            if (partIndex === itemParts.length) {
                // If no existing part, create a new one
                // TODO (justinfagnani): test perf impact of always creating two parts
                // instead of sharing parts between nodes
                // https://github.com/Polymer/lit-html/issues/1266
                itemParts.push((itemPart = new NodePart(this.__insert(createMarker()), this.__insert(createMarker()), this.options)));
            }
            else {
                // Reuse an existing part
                itemPart = itemParts[partIndex];
            }
            itemPart._setValue(item);
            partIndex++;
        }
        if (partIndex < itemParts.length) {
            // Truncate the parts array so _value reflects the current state
            itemParts.length = partIndex;
            // itemParts always have end nodes
            this.__clear(itemPart === null || itemPart === void 0 ? void 0 : itemPart._endNode.nextSibling);
        }
    }
    __clear(start = this._startNode.nextSibling) {
        while (start && start !== this._endNode) {
            const n = start.nextSibling;
            start.remove();
            start = n;
        }
    }
}
export class AttributePart {
    constructor(element, name, strings, _options) {
        this.type = ATTRIBUTE_PART;
        this._value = nothing;
        this.element = element;
        this.name = name;
        if (strings.length > 2 || strings[0] !== '' || strings[1] !== '') {
            this._value = new Array(strings.length - 1).fill(nothing);
            this.strings = strings;
        }
        else {
            this._value = nothing;
        }
    }
    get tagName() {
        return this.element.tagName;
    }
    /**
     * Normalizes a user-provided value before writing it to the DOM. In the
     * near future this will include invoking a directive if the value is
     * a DirectiveResult.
     *
     * @param value the raw input value to normalize
     * @param _i the index in the values array this value was read from
     */
    __resolveValue(value, i) {
        var _a, _b;
        const directiveCtor = (_a = value) === null || _a === void 0 ? void 0 : _a._$litDirective$;
        if (directiveCtor !== undefined) {
            // TODO (justinfagnani): Initialize array to the correct value,
            // or check length.
            let directive = ((_b = this.__directives) !== null && _b !== void 0 ? _b : (this.__directives = []))[i];
            if ((directive === null || directive === void 0 ? void 0 : directive.constructor) !== directiveCtor) {
                directive = this.__directives[i] = new directiveCtor(this);
            }
            // TODO (justinfagnani): To support nested directives, we'd need to
            // resolve the directive result's values. We may want to offer another
            // way of composing directives.
            value = directive.update(this, value.values);
        }
        return value !== null && value !== void 0 ? value : '';
    }
    _setValue(value, from) {
        const strings = this.strings;
        if (strings === undefined) {
            // Single-value binding case
            const v = this.__resolveValue(value, 0);
            // Only dirty-check primitives and `nothing`:
            // `(isPrimitive(v) || v === nothing)` limits the clause to primitives and
            // `nothing`. `v === this._value` is the dirty-check.
            if (!((isPrimitive(v) || v === nothing) && v === this._value) &&
                v !== noChange) {
                this.__commitValue((this._value = v));
            }
        }
        else {
            // Interpolation case
            let attributeValue = strings[0];
            // Whether any of the values has changed, for dirty-checking
            let change = false;
            // Whether any of the values is the `nothing` sentinel. If any are, we
            // remove the entire attribute.
            let remove = false;
            let i, v;
            for (i = 0; i < strings.length - 1; i++) {
                v = this.__resolveValue(value[from + i], i);
                if (v === noChange) {
                    // If the user-provided value is `noChange`, use the previous value
                    v = this._value[i];
                }
                else {
                    remove = remove || v === nothing;
                    change =
                        change ||
                            !((isPrimitive(v) || v === nothing) &&
                                v === this._value[i]);
                    this._value[i] = v;
                }
                attributeValue +=
                    (typeof v === 'string' ? v : String(v)) + strings[i + 1];
            }
            if (change) {
                this.__commitValue(remove ? nothing : attributeValue);
            }
        }
    }
    /**
     * Writes the value to the DOM. An override point for PropertyPart and
     * BooleanAttributePart.
     */
    __commitValue(value) {
        if (value === nothing) {
            this.element.removeAttribute(this.name);
        }
        else {
            this.element.setAttribute(this.name, value);
        }
    }
}
export class PropertyPart extends AttributePart {
    constructor() {
        super(...arguments);
        this.type = PROPERTY_PART;
    }
    __commitValue(value) {
        this.element[this.name] = value === nothing ? undefined : value;
    }
}
export class BooleanAttributePart extends AttributePart {
    constructor() {
        super(...arguments);
        this.type = BOOLEAN_ATTRIBUTE_PART;
    }
    __commitValue(value) {
        if (value && value !== nothing) {
            this.element.setAttribute(this.name, '');
        }
        else {
            this.element.removeAttribute(this.name);
        }
    }
}
/**
 * An AttributePart that manages an event listener via add/removeEventListener.
 *
 * This part works by adding itself as the event listener on an element, then
 * delegating to the value passed to it. This reduces the number of calls to
 * add/removeEventListener if the listener changes frequently, such as when an
 * inline function is used as a listener.
 *
 * Because event options are passed when adding listeners, we must take case
 * to add and remove the part as a listener when the event options change.
 */
export class EventPart extends AttributePart {
    constructor(...args) {
        var _a;
        super(...args);
        this.type = EVENT_PART;
        this.__eventContext = (_a = args[3]) === null || _a === void 0 ? void 0 : _a.eventContext;
    }
    _setValue(newListener) {
        newListener !== null && newListener !== void 0 ? newListener : (newListener = nothing);
        const oldListener = this._value;
        // If the new value is nothing or any options change we have to remove the
        // part as a listener.
        const shouldRemoveListener = (newListener === nothing && oldListener !== nothing) ||
            newListener.capture !==
                oldListener.capture ||
            newListener.once !==
                oldListener.once ||
            newListener.passive !==
                oldListener.passive;
        // If the new value is not nothing and we removed the listener, we have
        // to add the part as a listener.
        const shouldAddListener = newListener !== nothing &&
            (oldListener === nothing || shouldRemoveListener);
        if (shouldRemoveListener) {
            this.element.removeEventListener(this.name, this, oldListener);
        }
        if (shouldAddListener) {
            // Beware: IE11 and Chrome 41 don't like using the listener as the
            // options object. Figure out how to deal w/ this in IE11 - maybe
            // patch addEventListener?
            this.element.addEventListener(this.name, this, newListener);
        }
        this._value = newListener;
    }
    handleEvent(event) {
        var _a;
        if (typeof this._value === 'function') {
            // TODO (justinfagnani): do we need to default to this.__element?
            // It'll always be the same as `e.currentTarget`.
            this._value.call((_a = this.__eventContext) !== null && _a !== void 0 ? _a : this.element, event);
        }
        else {
            this._value.handleEvent(event);
        }
    }
}
// IMPORTANT: do not change the property name or the assignment expression.
// This line will be used in regexes to search for lit-html usage.
// TODO(justinfagnani): inject version number at build time
((_a = (_b = globalThis)['litHtmlVersions']) !== null && _a !== void 0 ? _a : (_b['litHtmlVersions'] = [])).push('2.0.0-pre.2');
//# sourceMappingURL=lit-html.js.map